<?php namespace Myriad\Installations\Middleware;

use Closure;
use Illuminate\Contracts\Validation\Factory;
use Illuminate\Support\Arr;
use Myriad\Installations\Exceptions\InstallationValidationException;
use Myriad\Installations\InstallationManager;

class VerifyInstallation {

  /**
   * @var InstallationManager the installation manager
   */
  private $manager;

  /**
   * @var Factory
   */
  private $validator;

  /**
   * Create a new middleware instance.
   *
   * @param InstallationManager $manager
   * @param Factory $validator
   */
  public function __construct(InstallationManager $manager, Factory $validator) {
    $this->manager = $manager;
    $this->validator = $validator;
  }
  
  /**
   * Handle an incoming request.
   *
   * @param  \Illuminate\Http\Request $request
   * @param  \Closure $next
   *
   * @return mixed
   */
  public function handle($request, Closure $next) {
    $params = $this->manager->getParamsFromRequest();

    $rules = Arr::only($this->manager->getValidator()->getRules(), array_keys($params));
    $messages = Arr::only($this->manager->getValidator()->getMessages(), array_keys($params));

    $validator = $this->validator->make($params, $rules, $messages);
    if ($validator->fails()) {
      throw new InstallationValidationException($validator);
    }

    $this->manager->createOrUpdateInstallation($this->manager->getUuidFromRequest(), $params);

    return $next($request);
  }

}